from fastapi import FastAPI
from estools import ESTools
from typing import Optional
from urllib.parse import unquote
import uvicorn

app = FastAPI()


@app.get("/comparison")
async def comparison(price: float, title: str, size: Optional[int] = 20, userid: Optional[str] = None):
    item_name = unquote(title, encoding='utf-8')
    es = ESTools()
    if not es:
        return {
            "code": 2005,
            "msg": "连接es错误"
        }
    else:
        res = es.data_format(price=price, title=item_name, size=size)
        if res:
            return {
                "code": 2000,
                "msg": "成功",
                "data": {
                    "size": len(res),
                    "list": res
                }
            }
        else:
            return {
                "code": 2004,
                "msg": "没有找到商品"
            }


if __name__ == '__main__':
    uvicorn.run(app='api:app', host="0.0.0.0", port=8000, reload=True, debug=True)
